
-------------------------------------------------------
--             Shunting Yard Algorithm              --
-------------------------------------------------------

--Takes String and processes into two tables of Expressions and Operators --
function Shunting_Yard(formula)

	local queue = {}
	local Op_Stack = {}

	local function Multi_Digits(string)
		local recall = "";
		for i = 1, #string do
			if type(tonumber(string:sub(i, i))) == "number" then
				recall = recall .. string:sub(i, i)
			else
				return recall
			end
		end
	end

	formula = formula:gsub("%s", "")

	while #formula > 0 do
		local Expr_Token = tonumber(Multi_Digits(formula))

		if Expr_Token ~= nil then
			queue[#queue + 1] = Expr_Token
		else
			local Op_Token = formula:sub(1, 1)
			local iter;
			if #Op_Stack == 0 then
				table.insert(Op_Stack, Op_Token)
			else
				while true do
					iter = #Op_Stack
					if Op_Token == "+" or Op_Token == "-" then
						while true do
							if Op_Stack[iter] == "+" or Op_Stack[iter] == "-" then
								table.insert(Op_Stack, iter + 1, Op_Token)
								break
							elseif Op_Stack[iter] == "(" or Op_Stack[iter] == ")" then
								table.insert(Op_Stack, iter + 1, Op_Token)
								break
							end
							iter = iter - 1
						end
						break
					elseif Op_Token == "/" or Op_Token == "*" then
						while true do
							if Op_Stack[iter] == "+" or Op_Stack[iter] == "-" or Op_Stack[iter] == "*" or Op_Stack[iter] == "/" then
								table.insert(Op_Stack, iter + 1, Op_Token)
								break
							elseif Op_Stack[iter] == "(" or Op_Stack[iter] == ")" then
								table.insert(Op_Stack, iter + 1, Op_Token)
								break
							end
							iter = iter - 1
						end
						break
					elseif Op_Token == "(" then
						table.insert(Op_Stack, #Op_Stack + 1, Op_Token)
						break
					elseif Op_Token == ")" then
						while true do
							if Op_Stack[iter] ~= "(" then
								local temp = table.remove(Op_Stack, iter)
								table.insert(queue, #queue + 1, temp)
							else
								Op_Stack[iter] = nil
								break
							end
							iter = iter - 1
						end
						break
					end
				end
			end
		end
		if type(tonumber(Multi_Digits(formula))) == "number" then
			formula = formula:sub(#Multi_Digits(formula) + 1, -1)
		else
			formula = formula:sub(2, -1)
		end
	end

	while #Op_Stack > 0 do
		table.insert(queue, #queue + 1, Op_Stack[#Op_Stack])
		table.remove(Op_Stack, #Op_Stack)
	end

	return queue
end

-------------------------------------------------------
--             Reverse Polish Notation               --
-------------------------------------------------------

--Takes one lists of tokens, and calculates the answer
function Reverse_Polish_Calc(queue)
	local stack = {}

	while #queue > 0 do
		local temp = table.remove(queue, 1)
		if type(tonumber(temp)) == "number" then
			table.insert(stack, #stack + 1, temp)
		else
			if temp == "+" then
				local x = table.remove(stack, #stack - 1)
				local y = table.remove(stack, #stack)
				table.insert(stack, #stack + 1, x + y)
			elseif temp == "-" then
				local x = table.remove(stack, #stack - 1)
				local y = table.remove(stack, #stack)
				table.insert(stack, #stack + 1, x - y)
			elseif temp == "*" then
				local x = table.remove(stack, #stack - 1)
				local y = table.remove(stack, #stack)
				table.insert(stack, #stack + 1, x * y)
			elseif temp == "/" then
				local x = table.remove(stack, #stack - 1)
				local y = table.remove(stack, #stack)
				table.insert(stack, #stack + 1, x / y)
			end
		end
	end

	return stack[1]
end


-------------------------------------------------------
--             String Library Calculator             --
-------------------------------------------------------

--To Do:
--Triginometry
--Logrithmic
--Faster factorial using table lookups rather than recursive function calling
--
--
--
--

function String_Library_Calc(formula, first)

	local value
	local newForm

	local factorial = {
		1, 2, 6, 24, 120, 720, 5040, 40320, 362880, 3628800, 39916800, 479001600
	}


	if formula:find("%a.") then
		error("Calculate cannot calculate variables.")
	end


	--If this is the first time the function runs
	--Takes away all spaces
	if first == nil or first then
		formula = formula:gsub("%s", "")
	end

	--Finds parenthesis
	--If found it will chop off the code around the parenthesis and take them away.
	--It will then create a recursion to allow multiple parethesis.
	while formula:match("%b()") do
		newForm = String_Library_Calc((formula:match("%b()")):sub(2, -2), false)
		formula = formula:gsub("%b()", newForm, 1)
	end

	if formula:find("!") then
		for word in formula:gmatch("!") do
			local x = formula:match("([-]?%d+%.*%d*)!")

			formula = formula:gsub("[-]?%d+%.*%d*!", factorial[tonumber(x)])
		end
	end

	--Find the Exponents, then the numbers next to them. And math.pow the left number by the right
	if formula:find("^") then
		local pow = math.pow
		for word in formula:gmatch("^") do
			local x, y = formula:match("([-]?%d+%.*%d*)^([-]?%d+%.*%d*)")

			formula = formula:gsub("[-]?%d+%.*%d*^[-]?%d+%.*%d*", pow(x, y))
		end
	end

	if formula:find("*") then
		for word in formula:gmatch("*") do
			local x, y = formula:match("([-]?%d+%.*%d*)*([-]?%d+%.*%d*)")

			formula = formula:gsub("[-]?%d+%.*%d**[-]?%d+%.*%d*", x * y)
		end
	end

	if formula:find("/") then
		for word in formula:gmatch("/") do
			local x, y = formula:match("([-]?%d+%.*%d*)/([-]?%d+%.*%d*)")
			if y == 0 then error("division by zero") end

			formula = formula:gsub("[-]?%d+%.*%d*/[-]?%d+%.*%d*", x / y)
		end
	end

	if formula:find("-") then
		for word in formula:gmatch("%-") do
			local x, y = formula:match("([-]?%d+%.*%d*)-([-]?%d+%.*%d*)")

			formula = formula:gsub("[-]?%d+%.*%d*-[-]?%d+%.*%d*", x - y)
		end
	end

	if formula:find("+") then
		for word in formula:gmatch("+") do
			local x, y = formula:match("([-]?%d+%.*%d*)+([-]?%d+%.*%d*)")

			formula = formula:gsub("[-]?%d+%.*%d*+[-]?%d+%.*%d*", x + y)
		end
	end

	return formula
end

-------------------------------------------------------
--               Load-String Function                --
-------------------------------------------------------